@@ -7,6 +7,7 @@ import subprocess
 import time
 from contextlib import contextmanager
 from datetime import datetime, timedelta
+from threading import Thread
 from typing import NamedTuple, Optional, Dict, List, Callable
 
 import yaml
@@ -14,8 +15,6 @@ from psycopg2._psycopg import DatabaseError
 from psycopg2.extras import NamedTupleCursor
 from psycopg2.pool import ThreadedConnectionPool
 
-from cortex_serving_client.log_thread import LogThread
-
 """
 Details: https://www.cortex.dev/deployments/statuses
 On some places custom "not_deployed" status may be used.
@@ -24,9 +23,10 @@ On some places custom "not_deployed" status may be used.
 CORTEX_STATUSES = ["live", "updating", "error", "error (out of memory)", "compute unavailable"]  #
 CORTEX_DELETE_TIMEOUT_SEC = 10 * 60
 CORTEX_DEPLOY_REPORTED_TIMEOUT_SEC = 60
-CORTEX_DEFAULT_DEPLOYMENT_TIMEOUT = 15 * 60
+CORTEX_DEFAULT_DEPLOYMENT_TIMEOUT = 20 * 60
 CORTEX_DEFAULT_API_TIMEOUT = CORTEX_DEFAULT_DEPLOYMENT_TIMEOUT
 INFINITE_TIMEOUT_SEC = 30 * 365 * 24 * 60 * 60  # 30 years
+CORTEX_DEFAULT_COMMAND_SYSTEM_PROCESS_TIMEOUT = 3 * 60
 
 
 logger = logging.getLogger(__name__)
@@ -237,7 +237,7 @@ class CortexClient:
         def listen_on_logs():
             os.system("cortex logs " + name + " " + f"--env={self.cortex_env}")
 
-        worker = LogThread(target=listen_on_logs, daemon=True)
+        worker = Thread(target=listen_on_logs, daemon=True)
         worker.start()
 
     @staticmethod
@@ -261,47 +261,51 @@ class CortexClient:
 
     def _collect_garbage(self):
         logger.info(f"Starting garbage collection.")
-        with self._open_db_cursor() as cur:
-            # remove timed out - deployed and recorded
-            cur.execute(
-                "select api_name from cortex_api_timeout where ultimate_timeout < transaction_timestamp() for update"
-            )
-            for api_row in cur.fetchall():
-                api_name = api_row.api_name
-                if self.get(api_name).status != "not_deployed":
-                    logger.warning(f"Collecting Cortex garbage - timed-out deployed API: {api_name}")
-                    self.delete(api_name)
-
-                else:
-                    logger.warning(f"Collecting Cortex garbage - timed-out db row: {api_name}")
-                    self._del_db_api_row(api_name, cur)
-
-            cur.execute("commit")
-
-            deployed_api_names = [row.api for row in self.get_all()]
-
-            # Remove recorded but not deployed - fast to avoid conflicts
-            cur.execute(
-                "select api_name from cortex_api_timeout where modified + %s * interval '1 second' < transaction_timestamp() and not (api_name = ANY(%s))",
-                [CORTEX_DEPLOY_REPORTED_TIMEOUT_SEC, deployed_api_names],
-            )
-            if cur.rowcount > 0:
-                apis = [r.api_name for r in cur.fetchall()]
-                logger.warning(f"Collecting Cortex garbage - recorded but not deployed: {apis}")
+        try:
+            with self._open_db_cursor() as cur:
+                # remove timed out - deployed and recorded
                 cur.execute(
-                    "delete from cortex_api_timeout where modified + %s * interval '1 second' < transaction_timestamp() and not (api_name = ANY(%s))",
-                    [CORTEX_DEPLOY_REPORTED_TIMEOUT_SEC, deployed_api_names],
+                    "select api_name from cortex_api_timeout where ultimate_timeout < transaction_timestamp() for update"
                 )
+                for api_row in cur.fetchall():
+                    api_name = api_row.api_name
+                    if self.get(api_name).status != "not_deployed":
+                        logger.warning(f"Collecting Cortex garbage - timed-out deployed API: {api_name}")
+                        self.delete(api_name, cursor=cur)
+
+                    else:
+                        logger.warning(f"Collecting Cortex garbage - timed-out db row: {api_name}")
+                        self._del_db_api_row(api_name, cur)
+
                 cur.execute("commit")
 
-            # Remove deployed but not recorded
-            cur.execute(
-                "select api_name from cortex_api_timeout", [CORTEX_DEPLOY_REPORTED_TIMEOUT_SEC],
-            )
-            recorded_apis_set = set([r.api_name for r in cur.fetchall()])
-            for deployed_not_recorded_name in set(deployed_api_names).difference(recorded_apis_set):
-                logger.warning(f"Collecting Cortex garbage - deployed not recorded: {deployed_not_recorded_name}")
-                self.delete(deployed_not_recorded_name)
+                deployed_api_names = [row.api for row in self.get_all()]
+
+                # Remove recorded but not deployed - fast to avoid conflicts
+                cur.execute(
+                    "select api_name from cortex_api_timeout where modified + %s * interval '1 second' < transaction_timestamp() and not (api_name = ANY(%s))",
+                    [CORTEX_DEPLOY_REPORTED_TIMEOUT_SEC, deployed_api_names],
+                )
+                if cur.rowcount > 0:
+                    apis = [r.api_name for r in cur.fetchall()]
+                    logger.warning(f"Collecting Cortex garbage - recorded but not deployed: {apis}")
+                    cur.execute(
+                        "delete from cortex_api_timeout where modified + %s * interval '1 second' < transaction_timestamp() and not (api_name = ANY(%s))",
+                        [CORTEX_DEPLOY_REPORTED_TIMEOUT_SEC, deployed_api_names],
+                    )
+                    cur.execute("commit")
+
+                # Remove deployed but not recorded
+                cur.execute(
+                    "select api_name from cortex_api_timeout", [CORTEX_DEPLOY_REPORTED_TIMEOUT_SEC],
+                )
+                recorded_apis_set = set([r.api_name for r in cur.fetchall()])
+                for deployed_not_recorded_name in set(deployed_api_names).difference(recorded_apis_set):
+                    logger.warning(f"Collecting Cortex garbage - deployed not recorded: {deployed_not_recorded_name}")
+                    self.delete(deployed_not_recorded_name, cursor=cur)
+
+        except Exception as e:
+            logger.warning(f"Ignoring unexpected exception that occurred during Garbage Collection: {e}", exc_info=e)
 
     def _init_garbage_api_collector(self, interval_sec):
         with self._open_db_cursor() as cur:
@@ -327,17 +331,13 @@ class CortexClient:
                 """
                 )
 
-        self.looping_thread = LogThread(target=lambda: self._start_gc_loop(interval_sec), daemon=True)
+        self.looping_thread = Thread(target=lambda: self._start_gc_loop(interval_sec), daemon=True)
         self.looping_thread.start()
 
     def _start_gc_loop(self, interval_sec):
         while True:
             time.sleep(interval_sec)
-            try:
-                self._collect_garbage()
-
-            except Exception as e:
-                logger.warning(f"WarningException occurred during garbage collection.", exc_info=e)
+            self._collect_garbage()
 
 
 @contextmanager
@@ -353,25 +353,35 @@ def open_pg_cursor(db_connection_pool, key=None):
 
 
 def _verbose_command_wrapper(
-    cmd_arr: List[str], cwd: str = None, timeout: int = None, input: bytes = None, retry_count=3
+        cmd_arr: List[str], cwd: str = None, timeout: int = CORTEX_DEFAULT_COMMAND_SYSTEM_PROCESS_TIMEOUT,
+        input: bytes = None, retry_count=3
 ):
     cmd_str = " ".join(cmd_arr)
     message = ""
     for retry in range(retry_count):
-        p = subprocess.run(cmd_arr, stdout=subprocess.PIPE, cwd=cwd, timeout=timeout, input=input)
-        out = p.stdout.decode()
-        # check_returncode() does not print process output to the console automatically so custom below
-        if p.returncode == 0:
-            logger.debug(f"Successful commmand {cmd_arr} stdout is {json.dumps(out)}")
-            return out
+        try:
+            p = subprocess.run(cmd_arr, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=cwd, timeout=timeout, input=input)
+            out = p.stdout.decode()
+            # check_returncode() does not print process output to the console automatically so custom below
+            if p.returncode == 0:
+                logger.debug(f"Successful commmand {cmd_arr} stdout is {json.dumps(out)}")
+                return out
 
-        message = f"Non zero return code for command {cmd_str}! Stdout:\n{json.dumps(out)}"
-        if retry <= retry_count:
-            logger.warning(message)
+            message = f"Non zero return code for command {cmd_str}! Stdout:\n{json.dumps(out)}"
+            if retry <= retry_count:
+                logger.warning(message)
+
+        except subprocess.TimeoutExpired as e:
+            timeout_message = f'{e} with stdout: "{e.output}" and stderr: "{e.stderr}"'
+            if retry <= retry_count:
+                logger.warning("Retry ignoring exception: " + timeout_message)
+
+            else:
+                raise ValueError("Retry count exceeded: " + timeout_message) from e
 
         time.sleep(3)
 
-    raise ValueError(message)
+    raise ValueError("Retry count exceeded: " + message)
 
 
 class CortexGetAllStatus(NamedTuple):
